import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:video_player/video_player.dart';

class VideoListWidget extends StatefulWidget {
  final String video;
  final bool isFullScreen;
  final int position;
  final Function(bool isFullScreen) onToggleScreen;
  final Function() onPlayNext;
  final Function() onPlayPrev;

  const VideoListWidget(
      {Key key,
      this.isFullScreen = false,
      this.video,
      this.position = 0,
      this.onPlayNext,
      this.onPlayPrev,
      @required this.onToggleScreen})
      : super(key: key);
  @override
  _VideoListWidgetState createState() => _VideoListWidgetState();
}

class _VideoListWidgetState extends State<VideoListWidget>
    with TickerProviderStateMixin<VideoListWidget> {
  AnimationController controlController;
  VideoPlayerController videoPlayerController;

  bool isControlsAllowed;
  bool isPlaying;
  bool isLoading;
  Duration start;
  Duration end;
  AnimationController sliderController;

  @override
  void initState() {
    super.initState();
    isControlsAllowed = true;
    isPlaying = false;
    isLoading = true;
    start = Duration(milliseconds: 0);
    end = Duration(milliseconds: 0);

    controlController =
        AnimationController(vsync: this, duration: Duration(milliseconds: 150));
    sliderController =
        AnimationController(vsync: this, duration: Duration(milliseconds: 1));
    controlController.forward();
    WidgetsBinding.instance.addPostFrameCallback((duration) {
      initialize();
    });
    SystemChrome.setEnabledSystemUIOverlays([]);
  }

  void initialize() async {
    setState(() {
      isLoading = true;
    });
    videoPlayerController = VideoPlayerController.network(
      widget.video,
    );
    await videoPlayerController.initialize();
    setState(() {
      isLoading = false;
      end = Duration(
          milliseconds: videoPlayerController.value.duration.inMilliseconds);
    });
    videoPlayerController.seekTo(Duration(milliseconds: widget.position));
    videoPlayerController.play();
    videoPlayerController.addListener(() async {
      if (!mounted) {
        return;
      }
      if (isPlaying != videoPlayerController.value.isPlaying) {
        setState(() {
          isPlaying = videoPlayerController.value.isPlaying;
        });
      }
      Duration position = await videoPlayerController.position;
      if (sliderController != null)
        sliderController.value = position.inMilliseconds / end.inMilliseconds;
    });
  }

  @override
  Widget build(BuildContext context) {
    if (videoPlayerController != null &&
        widget.video != videoPlayerController.dataSource) {
      videoPlayerController.dispose();
      initialize();
    }
    return LayoutBuilder(builder: (context, constrinats) {
      double videoHeight = constrinats.biggest.height;
      double videoWidth = constrinats.biggest.width;
      if (videoHeight == double.infinity) {
        videoHeight = 220;
      }
      return Hero(
        tag: "video",
        child: Material(
          color: Colors.transparent,
          child: Container(
            width: videoWidth,
            height: videoHeight,
            child: Stack(
              alignment: Alignment.center,
              // fit: StackFit.expand,
              children: <Widget>[
                if (videoPlayerController?.value?.initialized ?? false)
                  Center(
                      child: SizedBox(
                          child: AspectRatio(
                              aspectRatio:
                                  videoPlayerController.value.aspectRatio,
                              child: VideoPlayer(videoPlayerController))))
                else
                  Container(
                    color: Colors.black,
                  ),
                // Middle Layer
                GestureDetector(
                  onTap: () {
                    toggleView();
                  },
                  child: Container(
                    decoration: BoxDecoration(
                        color: !isControlsAllowed
                            ? Colors.transparent
                            : Colors.black26),
                  ),
                ),
                // Controls
                AnimatedBuilder(
                  animation: controlController,
                  builder: (context, child) {
                    return Visibility(
                      visible: isControlsAllowed,
                      child: Opacity(
                        child: child,
                        opacity: controlController.value,
                      ),
                    );
                  },
                  child: Container(
                    child: Column(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: <Widget>[
                        // Menu
                        Container(
                          alignment: Alignment.centerRight,
                          child: IconButton(
                            icon: Icon(
                              Icons.fullscreen,
                              color: Colors.white,
                            ),
                            onPressed: () async {
                              if (!widget.isFullScreen) {
                                widget.onToggleScreen(true);

                                SystemChrome.setPreferredOrientations([
                                  DeviceOrientation.landscapeLeft,
                                  DeviceOrientation.landscapeRight
                                ]);
                              } else {
                                widget.onToggleScreen(false);
                                SystemChrome.setPreferredOrientations([
                                  DeviceOrientation.portraitDown,
                                  DeviceOrientation.portraitUp
                                ]);
                              }
                            },
                          ),
                        ),
                        // Play Controls
                        isLoading
                            ? Center(
                                child: CircularProgressIndicator(),
                              )
                            : Row(
                                mainAxisSize: MainAxisSize.min,
                                children: <Widget>[
                                  buildActionButton(
                                      Icons.replay_5, 24, _replay),
                                  SizedBox(
                                    width: videoWidth * 0.05,
                                  ),
                                  buildActionButton(
                                      isPlaying
                                          ? Icons.pause
                                          : Icons.play_arrow,
                                      36, () async {
                                    if (isPlaying) {
                                      await videoPlayerController.pause();
                                    } else {
                                      if (end == start) {
                                        await videoPlayerController
                                            .seekTo(Duration(milliseconds: 0));
                                      }
                                      await videoPlayerController.play();
                                    }
                                  }),
                                  SizedBox(
                                    width: videoWidth * 0.05,
                                  ),
                                  buildActionButton(
                                      Icons.forward_5, 24, _forward),
                                ],
                              ),
                        // Position Controls
                        AnimatedBuilder(
                            animation: sliderController,
                            child: Container(),
                            builder: (context, child) {
                              return Column(
                                mainAxisSize: MainAxisSize.min,
                                children: <Widget>[
                                  Padding(
                                    padding: const EdgeInsets.symmetric(
                                        horizontal: 12),
                                    child: Row(
                                      mainAxisAlignment:
                                          MainAxisAlignment.spaceBetween,
                                      children: <Widget>[
                                        Text(
                                          DateHelper.getAudioDuration(Duration(
                                              milliseconds:
                                                  (sliderController.value *
                                                          end.inMilliseconds)
                                                      .toInt())),
                                          style: style,
                                        ),
                                        Text(
                                          DateHelper.getAudioDuration(end),
                                          style: style,
                                        ),
                                      ],
                                    ),
                                  ),
                                  SizedBox(
                                    child: Slider(
                                      value: sliderController.value,
                                      onChanged: (val) {
                                        videoPlayerController.seekTo(Duration(
                                            milliseconds:
                                                (val * end.inMilliseconds)
                                                    .toInt()));
                                      },
                                      activeColor: Colors.redAccent,
                                      inactiveColor: Colors.white,
                                    ),
                                  ),
                                ],
                              );
                            })
                      ],
                    ),
                  ),
                )
              ],
            ),
          ),
        ),
      );
    });
  }

  Future<void> _forward() async {
    await videoPlayerController
        .seekTo(videoPlayerController.value.position + Duration(seconds: 5));
  }

  Future<void> _replay() async {
    await videoPlayerController
        .seekTo(videoPlayerController.value.position - Duration(seconds: 5));
  }

  double get sliderPostion => start.inMilliseconds / end.inMilliseconds;

  Widget buildActionButton(
      IconData iconData, double size, Function() onPressed) {
    return EaseInWidget(
      onTap: onPressed,
      child: Opacity(
        opacity: onPressed != null ? 1 : 0.7,
        child: Container(
          decoration: BoxDecoration(
            shape: BoxShape.circle,
            color: Colors.white.withOpacity(0.25),
          ),
          padding: EdgeInsets.all(size / 3),
          alignment: Alignment.center,
          child: Icon(
            iconData,
            color: Colors.white,
            size: size,
          ),
        ),
      ),
    );
  }

  TextStyle get style => TextStyle(
      color: Colors.white, fontSize: 13, fontWeight: FontWeight.normal);

  void toggleView() async {
    if (isControlsAllowed) {
      await controlController.reverse();
    } else {
      await controlController.forward();
    }
    isControlsAllowed = !isControlsAllowed;
    if (mounted) setState(() {});
  }

  @override
  void deactivate() {
    super.deactivate();
    SystemChrome.setEnabledSystemUIOverlays([
      SystemUiOverlay.bottom,
      SystemUiOverlay.top,
    ]);
  }

  @override
  void dispose() {
    controlController.dispose();
    videoPlayerController.dispose();
    sliderController.dispose();
    sliderController = null;
    SystemChrome.setPreferredOrientations(
        [DeviceOrientation.portraitDown, DeviceOrientation.portraitUp]);
    super.dispose();
  }
}
